#include "mcsEnsemble.h"
#include <dirent.h>

extern "C"
{
	#include <ViennaRNA/fold_vars.h>
	char  *random_string(int l, const char symbols[]);
	float  fold(const char *sequence, char *structure);
	void   init_rand(void);
	int    PS_rna_plot(char *string, char *structure, char *file);
	int    PS_rna_plot_a(char *string, char *structure, char *file, char *pre, char *post);
}

void mcsEnsemble::clearEnsemble()
{
   ensembleMolecules.clear();
   ensemblePatterns = PatternPairMap();
}

void mcsEnsemble::load_input_data(const vector<string>& my_input)
{
	clearEnsemble();
	Molecule myMol1,myMol2;

	cout <<"###############################################################################"<<endl;
   cout << "    expaRNA - Exact Pattern Alignment of RNA  - version 0.5"<<endl;
   cout << "           (c) copyright by Steffen Heyne - 2008"<<endl;
   cout <<"###############################################################################"<<endl<<endl;
   cout << " --> Creating virtual molecules with provided data...";

   readFASTA(my_input[0],myMol1,myMol2);
 /*  if (my_input.size()==1)
      readFileInput2(my_input[0],myMol1,myMol2);

   if ((my_input.size()==2)&& (ensOptions.alignColor))
   {
         readFileInput2(my_input[0],myMol1,myMol2);
   }
   if ((my_input.size()==6) && (my_input[2]!="rand") && (my_input[5]!="rand"))
	{
     myMol1 = Molecule(my_input[0],my_input[1],my_input[0]+"_mod1",my_input[2]);
	}
*/
  /* if ((my_input.size()==6) && (my_input[2]=="rand") && (my_input[5]=="rand"))
	{
	  init_rand();
	  char my[5]="ACGU";
	  int len1=atoi(my_input[1].c_str());
	  char *struct1 = new char[len1];
	  char *tmp = random_string(len1,my);
	  fold(tmp, struct1);
     myMol1 = Molecule(my_input[0],tmp,my_input[0]+"_mod1",struct1);

	  int len2=atoi(my_input[4].c_str());
	  char* tmp2 = random_string(len2,my);
	  char *struct2 = new char[len2];
	  fold(tmp2, struct2);
     myMol2 = Molecule(my_input[3],tmp2,my_input[3]+"_mod1",struct2);
     cout.flush();
	}*/
   cout << "finished!" << endl << endl;
   if (ensOptions.verboseOut) myMol1.printInfo(false);
   if (ensOptions.verboseOut) myMol2.printInfo(false);
   ensOptions.mol1Id = myMol1.getId();
   ensOptions.mol2Id = myMol2.getId();
   ensembleMolecules.insert(make_pair(myMol1.getId(),myMol1));
   ensembleMolecules.insert(make_pair(myMol2.getId(),myMol2));
}

void mcsEnsemble::readFASTA(string myFASTAfile, Molecule& myMol1, Molecule& myMol2)
{
	std::ifstream fa_file(myFASTAfile.c_str(),ios::in);

	string line,seq1,seq1_name,seq1_str,seq1_strname,seq2,seq2_name,seq2_str,seq2_strname;

	dangles = 2; // vienna package variable for dangling ends, 2 standard for partion function folding

	if (fa_file)
	{
		getline(fa_file,line);

		if (line[0]=='>')
		{
			int tmppos = 1;
			while ((line[tmppos] != ' ') && (tmppos<line.size()))
			{
				seq1_name.push_back(line[tmppos]);
				++tmppos;
			}
		}
		else
		{
			cerr <<" Wrong fasta format!" << endl; exit(1);
		}

		getline(fa_file,seq1);
		getline(fa_file,line);
		if (line[0] != '>')
		{
			seq1_str = line;
			getline(fa_file,line);
			if (line[0]=='>')
			{
				int tmppos = 1;
				while ((line[tmppos] != ' ') && (tmppos<line.size()))
				{
					seq2_name.push_back(line[tmppos]);
					++tmppos;
				}
			}
			else
			{
				cerr << seq1_str << line << " wrong fasta format!" << endl; exit(1);
			}
		}
		else
		{
			if (line[0] =='>')
			{

				int tmppos = 1;
				while ((line[tmppos] != ' ') && (tmppos<line.size()))
				{
					seq2_name.push_back(line[tmppos]);
					++tmppos;
				}
			}
			else
			{
				cerr <<" wrong fasta format!" << endl; exit(1);
			}
		}
		fa_file >> seq2;
		if (!fa_file.eof()) { fa_file >> seq2_str; }
		fa_file.close();

		if (seq1_str.length()==0)
		{
			int len1=seq1.length();
		    char *c_str1 = new char[len1+1];
		    float my = fold(seq1.c_str(), c_str1);
		    seq1_str = (string)c_str1;
		    seq1_strname = seq1_name + "_mfe";
		}
		else
			seq1_strname=seq1_name+"_str";

		if (seq2_str.length()==0)
		{
			int len2=seq2.length();
			char *c_str2 = new char[len2+1];
			float my = fold(seq2.c_str(), c_str2);
			seq2_str = (string)c_str2;
			seq2_strname=seq2_name+"_mfe";
		}
		else
			seq2_strname=seq2_name+"_str";

		//cout << "\nseq1 name:" << seq1_name << "!" << endl << "seq1:" << seq1 << "!" << endl << "str1name:" << seq1_strname << endl << "str1:" << seq1_str << "!" << endl;
		//cout << "seq2 name:" << seq2_name << "!" << endl << "seq2:" << seq2 << "!" << endl <<  "str2name:" << seq2_strname << endl << "str2:" << seq2_str << "!" << endl;
		myMol1 = Molecule(seq1_name,seq1,seq1_strname,seq1_str);
		myMol2 = Molecule(seq2_name,seq2,seq2_strname,seq2_str);
	}
	else
    {
       cerr << "Could not open File:" << myFASTAfile << endl;
       exit(1);
    }
	fa_file.close();
}

string mcsEnsemble::getNext(std::ifstream &myFile, string& what, string& name)
{
   char buf[100];
   char c_name[200];
   char c_what[100];
   string data="";
   if (myFile.good())
   {
      if (myFile.good()) myFile.getline(buf,100,'<');
      if (myFile.good()) myFile.getline(c_what,100,'>');
      if (myFile.good()) myFile.getline(c_name,200,'\n');

      what = (string)c_what;
      name = (string)c_name;

      if (name=="")
      {
         cerr << "Please provide a name for "<<what<< " in input file " << myFile <<"!"<<endl;
         exit(1);
      }

      char c=' ';

      string tempData;
      while((c!='<')&&(myFile.good()&&(!myFile.eof())))
         {
            myFile.get(c);
            data = data + c;
            c=myFile.peek();
      }
      for (unsigned int i=0;i<name.length();i++)
         if (isgraph(name[i])) tempData += name[i];
      name = tempData;
      tempData.clear();

      if (what=="sequence")
        {
         for (unsigned int i=0;i<data.length();i++)
            if (isalpha(data[i])) tempData += toupper(data[i]);
        }
      else
         for (unsigned int i=0;i<data.length();i++)
            if (isgraph(data[i])) tempData += data[i];
      data = tempData;
   }
   return data;
}

void mcsEnsemble::readFileInput2(string myFile1,Molecule& myMol1,Molecule& myMol2)
{
   std::ifstream file1(myFile1.c_str());
   if (file1)
   {
      bool err = false;
      string seq1,seq2,str1,str2,name,seq1name,seq2name,str1name,str2name,next;
      string type = "";

      next = getNext(file1,type,name);
      if ((type=="sequence")&&(next!=""))
      {
         seq1=next;
         seq1name=name;
         type="";next="";
         next = getNext(file1,type,name);
         if ((type=="structure")&&(next!=""))
         {
            str1name=name;
            str1=next;
            next = getNext(file1,type,name);
         }
         else
         {
            int len1=seq1.length();
            char *c_str1 = new char[len1];
            fold(seq1.c_str(), c_str1);
            str1 = (string)c_str1;
            str1name=seq1name+"_RNAfold";
         }
      }
      if ((type=="sequence")&&(next!=""))
      {
         seq2=next;
         seq2name=name;
         type="";next="";
         next = getNext(file1,type,name);
         if ((type=="structure")&&(next!=""))
         {
            str2name=name;
            str2=next;
         }
         else
         {
            int len2=seq2.length();
            char *c_str2 = new char[len2];
            fold(seq2.c_str(), c_str2);
            str2 = (string)c_str2;
            str2name=seq2name+"_RNAfold";
         }
      }
      else
         err = true;
      if ((seq1.length()==str1.length())&&(seq2.length()==str2.length()))
      {
         myMol1 = Molecule(seq1name,seq1,str1name,str1);
         myMol2 = Molecule(seq2name,seq2,str2name,str2);
      }
      else
         err=true;
      if (err)
         {
              cout << "Error while reading input file!" << endl;
              exit(1);
         }
   }//end if file1
   else
      {
         cerr << "Could not open File:" << myFile1 <<endl;
         exit(1);
      }
   file1.close();
}

// update the molecules and the ensemblePatterns with new Patterns
/*void mcsEnsemble::updateEnsemblePatterns(const PatternPairMap& myPairMap)
{
   ensemblePatterns = myPairMap;
   for (MolMapITER j= ensembleMolecules.begin(); j!= ensembleMolecules.end();j++)
      {
         for(csPMapIter i = mySingMap.getMap().begin(); i!= mySingMap.getMap().end();i++)
         {
        //    ensemblePatterns.add(i->second);
            j->second.add(i->second);
         }
         cout << "Mol: " << j->second.getId()<< " contains " << j->second.moleculePatterns.getMap().size() << " patterns." << endl;
      }
}*/

void mcsEnsemble::printPatternMap()
{
 /*  for (cpMapIter i=ensemblePatterns.getMap().begin();i!=ensemblePatterns.getMap().end();i++)
   {
      Pattern myP =i->second;
      for (csPMapIter j=myP.getMap().begin();j!=myP.getMap().end();j++)
      {
         SinglePattern my = j->second;
         my.print();
      }
   }*/
}

void mcsEnsemble::startALL()
{
   struct timeval *Tps, *Tpf;
   struct timezone *Tzp;
   Tps = (struct timeval*) malloc(sizeof(struct timeval));
   Tpf = (struct timeval*) malloc(sizeof(struct timeval));
   Tzp = 0;

   // pattern finding
   gettimeofday (Tps, Tzp);
   mcsPatternFinding PairMCS(ensembleMolecules.find(ensOptions.mol1Id)->second,ensembleMolecules.find(ensOptions.mol2Id)->second,ensemblePatterns,ensOptions);
   PairMCS.start();
   gettimeofday (Tpf, Tzp);
   long double time = ((Tpf->tv_sec-Tps->tv_sec)*1000000 + Tpf->tv_usec-Tps->tv_usec)/1000000.0;
   cout << "time to find EPMs in secs: " << setprecision(4) << time << endl;
   if (ensOptions.verboseOut) cout << "The EPM library contains " << ensemblePatterns.size() << " EPMs." << endl;

   //LCSEPM
   if (ensOptions.calcLCSEPM)
	   startERPAP();

   // extract Alignment matchings for Coloring
   if (ensOptions.alignColor)
	   extractMatchings();

   // chdir to output
   cout << endl << " -->> start writing output" << endl;
   //string dirname = "output";
   //chdir(const_cast<char*>(dirname.c_str()));

   // postscript output
   if ((ensOptions.calcLCSEPM) && (ensOptions.clusterSaveCounter>=1))
   {
	   cout << "    -->> Writing LCS-EPM as Postscript file...";
	   clusterToPS(ensembleMolecules.find(ensOptions.mol1Id)->second,
	               ensembleMolecules.find(ensOptions.mol2Id)->second,
	               "LCSEPM",sizeERPAP,ensembleMaxERPAP);
	   cout << "  LCS-EPM output done!"<<endl;
   }
   cout << " -->> end writing output" << endl;

   // output edges of LCS-EPM and score etc
   if (ensOptions.locarnaOut)
   	   output_LocARNA();

   if (ensOptions.alignOut)
	   output_Clustal();

   if (ensOptions.epmOut)
   	   output_LCSEPM();

}

void mcsEnsemble::startERPAP()
{
	struct timeval *Tps, *Tpf;
	struct timezone *Tzp;
	Tps = (struct timeval*) malloc(sizeof(struct timeval));
	Tpf = (struct timeval*) malloc(sizeof(struct timeval));
	Tzp = 0;

	gettimeofday (Tps, Tzp);
	cout << endl << " -->> starting LCS-EPM algorithm" << endl;
	if (ensOptions.verboseOut) cout << "   EPMs for LCS-EPM: " << ensemblePatterns.size() << endl;
	mcsERPAP myERPAP(	ensembleMolecules.find(ensOptions.mol1Id)->second,
						ensembleMolecules.find(ensOptions.mol2Id)->second,
						ensemblePatterns,
						ensembleMaxERPAP,
						sizeERPAP,
						ensOptions);

	myERPAP.calculateERPAP();

	gettimeofday (Tpf, Tzp);
	long double time = ((Tpf->tv_sec-Tps->tv_sec)*1000000 + Tpf->tv_usec-Tps->tv_usec)/1000000.0;
	cout << "   time for LCS-EPM in secs: " << setprecision(4) << time << endl;
	cout << " -->> end LCS-EPM algorithm" << endl;
}


/*void mcsEnsemble::writeClustersAsPS(const int& myClusterSaveCount,const bool writeXML)
{
   cout << "    -->> Writing clusters as postscript files...";
   int myCounter = 1;
   for (mcsClustering::ClusterMapCITER myClus = ensembleClusters.begin();myClus != ensembleClusters.end();++myClus)
   {
      PatternPairMap clusterPatterns;
      for (mcsClustering::PatternKeyListCITER myPat = (*myClus).second->clusterPatternList.begin();myPat != (*myClus).second->clusterPatternList.end();++myPat)
      {
         clusterPatterns.add(ensemblePatterns.getPatternPairPTR((*myPat)->patternId));
      }
      string optStr;
      optStr = (*myClus).second->clusterId+"_c"+biu::Util_String::int2str(ensOptions.CLUSTER_METHODE)
               +"s"+ biu::Util_String::int2str(ensOptions.minPatternSize)
               +"d"+ biu::Util_String::int2str(ensOptions.DISTANCE_METHODE)
               +"t"+ biu::Util_String::int2str(ensOptions.distanceThresholdClustering)
               +"q"+ biu::Util_String::int2str(ensOptions.distanceEqualThreshold);
      clusterToPS(ensembleMolecules.find(ensOptions.mol1Id)->second,
                  ensembleMolecules.find(ensOptions.mol2Id)->second,
                  optStr,
                  (*myClus).second->clusterSize,
                  clusterPatterns);
      if (writeXML)
      {
    	 cout << endl<<"    -->> Writing clusters as XML files...";
         mcs_xml_io myxml_io("RNAPatterns");
         myxml_io.xml_append_molecule(ensembleMolecules.find(ensOptions.mol1Id)->second);
         myxml_io.xml_append_molecule(ensembleMolecules.find(ensOptions.mol2Id)->second);
         myxml_io.xml_append_patterns(clusterPatterns);
         myxml_io.xml_filewriter((*myClus).second->clusterId + "_"+biu::Util_String::int2str((*myClus).second->clusterSize) + "_" + ensOptions.mol1Id+ "_" + ensOptions.mol2Id + ".xml");
      }
      myCounter++;
      if (myCounter>myClusterSaveCount) break;
   }
   cout << "   output clustering done!" << endl;
}*/

/*void mcsEnsemble::writePatternMapAsPS(const Molecule& myMol1,
                                      const Molecule& myMol2,
                                      const PatternPairMap& myMap)
{
	cout << "Writing postscript files...";

	string func_str="\
	/drawpattern {\n\
		/Panz pattern length def\n\
		0 1 pattern length 1 sub {\n\
			/i exch def\n\
			pattern i get\n\
			newpath\n\
			{\n\
				1 Panz div i mul 0 add 1 1 sethsbcolor\n\
				coor exch 1 sub get aload pop fsize 2.1 div 0 360 arc\n\
				fill\n\
			} forall\n\
  		} for\n\
	} bind def\n\
	\n\
	/pattern [\n";
	string pat1_str,pat2_str;
   ID myID;
	for (PatternPairMap::patListCITER i=myMap.getList().begin();i != myMap.getList().end();i++)
	{
      //myID = ID(myMol1.getId(),myMol1.getActualStructureId(),i->getId());
      //vector <int> tmpvec1=i->second.getSinglePattern(myID).getPat();
      vector <int> tmpvec1=(*i)->getFirstPat().getPat();
	   pat1_str+="["+biu::Util_String::intvec2str(tmpvec1," ")+"]\n";
      //myID = ID(myMol2.getId(),myMol2.getActualStructureId(),i->getId());
      //vector <int> tmpvec2=i->second.getSinglePattern(myID).getPat();
      vector <int> tmpvec2=(*i)->getSecPat().getPat();
      pat2_str+="["+biu::Util_String::intvec2str(tmpvec2," ")+"]\n";
	}
	pat1_str+="] def\n\n";
   pat2_str+="] def\n\n";
	pat1_str=func_str+pat1_str;
	pat2_str=func_str+pat2_str;
 	string psfilename=myMol1.getActualStructureId()+"_pat.ps";
   PS_rna_plot_a(const_cast<char*>(myMol1.getSequence().c_str()),
 	              const_cast<char*>(myMol1.getStructure().c_str()),
 	              const_cast<char*>(psfilename.c_str()),
 	              const_cast<char*>(pat1_str.c_str()),"drawpattern\ndrawbases\n");

   psfilename=myMol2.getActualStructureId()+"_pat.ps";
   PS_rna_plot_a(const_cast<char*>(myMol2.getSequence().c_str()),
                 const_cast<char*>(myMol2.getStructure().c_str()),
                 const_cast<char*>(psfilename.c_str()),
                 const_cast<char*>(pat2_str.c_str()),"drawpattern\ndrawbases\n");

   cout << "done!" << endl;
} */

void mcsEnsemble::clusterToPS(const Molecule& myMol1,
                              const Molecule& myMol2,
                             /* const mcsClustering::ClusterInfoPTR& myClus,*/
                              const string&	  myId,
                              const int&      mySize,
                              const PatternPairMap& myMap)
{

   string func_str="\
   /drawpattern {\n\
      /Panz pattern length def\n\
      0 1 pattern length 1 sub {\n\
         /i exch def\n\
         pattern i get\n\
         newpath\n\
         {\n\
            1 Panz div i mul 0 add 1 1 sethsbcolor\n\
            coor exch 1 sub get aload pop fsize 2.1 div 0 360 arc\n\
            fill\n\
         } forall\n\
      } for\n\
   } bind def\n\
   \n\
   /pattern [\n";
   string clus1_str,clus2_str;
   ID myID;

  /* DIR *dirPTR;
   dirPTR=opendir("./postscript");
   if (dirPTR =NULL)
      cout << "No such dir!"<<endl;*/

   string label1Str,label2Str;

   for (unsigned int i=1;i<=myMol1.getSequence().length();++i)
   {
      if (i % 50 == 0)
         label1Str += biu::Util_String::int2str(i) + " 0.5 0.5 (" + biu::Util_String::int2str(i) + ") Label\n";
   }

   for (unsigned int i=1;i<=myMol2.getSequence().length();++i)
   {
      if (i % 50 == 0)
         label2Str += biu::Util_String::int2str(i) + " 0.5 0.5 (" + biu::Util_String::int2str(i) + ") Label\n";
   }

   for (PatternPairMap::patListCITER i=myMap.getList().begin();i != myMap.getList().end();i++)
   {
      intVec tmpvec1=(*i)->getFirstPat().getPat();
      clus1_str+="["+biu::Util_String::intvec2str(tmpvec1," ")+"]\n";
      intVec tmpvec2=(*i)->getSecPat().getPat();
      clus2_str+="["+biu::Util_String::intvec2str(tmpvec2," ")+"]\n";
   }
   clus1_str+="] def\n\n";
   clus2_str+="] def\n\n";
   clus1_str=func_str+clus1_str;
   clus2_str=func_str+clus2_str;

   string psfilename = myId + "_"+biu::Util_String::int2str(mySize) + "_" + myMol1.getActualStructureId()+".ps";
   PS_rna_plot_a(const_cast<char*>(myMol1.getSequence().c_str()),
                 const_cast<char*>(myMol1.getStructure().c_str()),
                 const_cast<char*>(psfilename.c_str()),
                 const_cast<char*>(clus1_str.c_str()),
                 const_cast<char*>(string("drawpattern\ndrawbases\n"+label1Str).c_str()));

   psfilename = myId + "_"+biu::Util_String::int2str(mySize) + "_" + myMol2.getActualStructureId()+".ps";
   PS_rna_plot_a(const_cast<char*>(myMol2.getSequence().c_str()),
                 const_cast<char*>(myMol2.getStructure().c_str()),
                 const_cast<char*>(psfilename.c_str()),
                 const_cast<char*>(clus2_str.c_str()),
                 const_cast<char*>(string("drawpattern\ndrawbases\n"+label2Str).c_str()));
}

void	mcsEnsemble::output_LocARNA()
{
	// extract matching edges (pairs of positions) from LCS-EPM
	vector<intPair> matchingsLCSEPM;
	intVec positionsSeq1LCSEPM;
	intVec positionsSeq2LCSEPM;

	for (PatternPairMap::patListCITER i=ensembleMaxERPAP.getList().begin();i != ensembleMaxERPAP.getList().end();i++)
	{
		positionsSeq1LCSEPM.insert(positionsSeq1LCSEPM.end(),(*i)->getFirstPat().getPat().begin(),(*i)->getFirstPat().getPat().end());
		positionsSeq2LCSEPM.insert(positionsSeq2LCSEPM.end(),(*i)->getSecPat().getPat().begin(),(*i)->getSecPat().getPat().end());
		//SinglePattern my1 = (*i)->getFirstPat();
		//my1.print();
		//my1 = (*i)->getSecPat();
		//my1.print();
	}

	sort(positionsSeq1LCSEPM.begin(),positionsSeq1LCSEPM.end());
	sort(positionsSeq2LCSEPM.begin(),positionsSeq2LCSEPM.end());;

	for (unsigned int i=0;i<positionsSeq1LCSEPM.size();++i)
	{
		matchingsLCSEPM.push_back(make_pair(positionsSeq1LCSEPM[i],positionsSeq2LCSEPM[i]));
	}

	Molecule& myMol1 = ensembleMolecules.find(ensOptions.mol1Id)->second;
	Molecule& myMol2 = ensembleMolecules.find(ensOptions.mol2Id)->second;

	ofstream outLocARNAfile (ensOptions.locarna_file.c_str());

	int last_edge_seq1,last_edge_seq2;
	last_edge_seq1=0;
	last_edge_seq2=0;

	string seq1_1,seq1_2,seq1_3,seq2_1,seq2_2,seq2_3;
	int edge = 100;

	for (vector<intPair>::iterator i_edge = matchingsLCSEPM.begin(); i_edge != matchingsLCSEPM.end();++i_edge)
	{
		//cout << "first: " << (*i_edge).first << " second: " << (*i_edge).second << endl;

		for (int i=last_edge_seq1+1;i<(*i_edge).first;++i)
		{
			seq1_1.push_back('.');
			seq1_2.push_back('.');
			seq1_3.push_back('.');
		}

		for (int j=last_edge_seq2+1;j<(*i_edge).second;++j)
		{
			seq2_1.push_back('.');
			seq2_2.push_back('.');
			seq2_3.push_back('.');
		}

		ostringstream edge_st_;
		edge_st_ << edge;
		string edge_st;
		edge_st = edge_st_.str();
		const char *c_str_edge = edge_st.c_str();

		seq1_1.push_back(c_str_edge[0]);
		seq1_2.push_back(c_str_edge[1]);
		seq1_3.push_back(c_str_edge[2]);

		seq2_1.push_back(c_str_edge[0]);
		seq2_2.push_back(c_str_edge[1]);
		seq2_3.push_back(c_str_edge[2]);

		++edge;

		last_edge_seq1= (*i_edge).first;
		last_edge_seq2 = (*i_edge).second;
	}

	// end stuff
	for (int i=last_edge_seq1+1;i<=myMol1.getLength();++i)
	{
		seq1_1.push_back('.');
		seq1_2.push_back('.');
		seq1_3.push_back('.');
	}

	for (int j=last_edge_seq2+1;j<=myMol2.getLength();++j)
	{
		seq2_1.push_back('.');
		seq2_2.push_back('.');
		seq2_3.push_back('.');
	}

	seq1_1 += " #1";
	seq1_2 += " #2";
	seq1_3 += " #3";

	seq2_1 += " #1";
	seq2_2 += " #2";
	seq2_3 += " #3";

	outLocARNAfile << ">" << myMol1.getId() << endl << myMol1.getSequence() << endl;
	outLocARNAfile << seq1_1 << endl << seq1_2 << endl << seq1_3 << endl;
	outLocARNAfile << ">" << myMol2.getId() << endl << myMol2.getSequence() << endl;
	outLocARNAfile << seq2_1 << endl << seq2_2 << endl << seq2_3 << endl;
	outLocARNAfile << myMol1.getLength() << " " << myMol1.getSequence().length() << endl;
	outLocARNAfile << myMol2.getLength() << " " << myMol2.getSequence().length() << 	endl;;
}

void	mcsEnsemble::output_LCSEPM()
{
	// extract matching edges (pairs of positions) from LCS-EPM
	vector<intPair> matchingsLCSEPM;
	intVec positionsSeq1LCSEPM;
	intVec positionsSeq2LCSEPM;

	Molecule& myMol1 = ensembleMolecules.find(ensOptions.mol1Id)->second;
	Molecule& myMol2 = ensembleMolecules.find(ensOptions.mol2Id)->second;

	ofstream outfile (ensOptions.epm_file.c_str());

	outfile << myMol1.getId() << endl;
	outfile << myMol1.getSequence() << endl;
	outfile << myMol1.getStructure() << endl;
	outfile << myMol2.getId() << endl;
	outfile << myMol2.getSequence() << endl;
	outfile << myMol2.getStructure() << endl;
	outfile << endl;

	for (PatternPairMap::patListCITER i=ensembleMaxERPAP.getList().begin();i != ensembleMaxERPAP.getList().end();i++)
	{
		positionsSeq1LCSEPM.assign((*i)->getFirstPat().getPat().begin(),(*i)->getFirstPat().getPat().end());
		positionsSeq2LCSEPM.assign((*i)->getSecPat().getPat().begin(),(*i)->getSecPat().getPat().end());

		string structure;
		string sequence;

		for (unsigned int j=0;j<positionsSeq1LCSEPM.size();++j)
		{
			structure += myMol1.getStructure(positionsSeq1LCSEPM[j]);
			sequence  += myMol1.getBase(positionsSeq1LCSEPM[j]);
		}

		outfile << myMol1.getId() << " " << myMol1.getId() << " " << (*i)->getId() << " " << sequence << " " << structure;
		outfile << " PosRNA1 " << biu::Util_String::intvec2str(positionsSeq1LCSEPM,":");
		outfile << " PosRNA2 " << biu::Util_String::intvec2str(positionsSeq2LCSEPM,":");
		outfile << endl;

		//SinglePattern my1 = (*i)->getFirstPat();
		//my1.print();
		//my1 = (*i)->getSecPat();
		//my1.print();
	}

	outfile.close();
}

void	mcsEnsemble::output_Clustal()
{
	// extract matching edges (pairs of positions) from LCS-EPM
	vector<intPair> matchingsLCSEPM;
	intVec positionsSeq1LCSEPM;
	intVec positionsSeq2LCSEPM;

	for (PatternPairMap::patListCITER i=ensembleMaxERPAP.getList().begin();i != ensembleMaxERPAP.getList().end();i++)
	{
		positionsSeq1LCSEPM.insert(positionsSeq1LCSEPM.end(),(*i)->getFirstPat().getPat().begin(),(*i)->getFirstPat().getPat().end());
		positionsSeq2LCSEPM.insert(positionsSeq2LCSEPM.end(),(*i)->getSecPat().getPat().begin(),(*i)->getSecPat().getPat().end());
	}

	sort(positionsSeq1LCSEPM.begin(),positionsSeq1LCSEPM.end());
	sort(positionsSeq2LCSEPM.begin(),positionsSeq2LCSEPM.end());;

	for (unsigned int i=0;i<positionsSeq1LCSEPM.size();++i)
	{
		matchingsLCSEPM.push_back(make_pair(positionsSeq1LCSEPM[i],positionsSeq2LCSEPM[i]));
	}

	Molecule& myMol1 = ensembleMolecules.find(ensOptions.mol1Id)->second;
	Molecule& myMol2 = ensembleMolecules.find(ensOptions.mol2Id)->second;

	ofstream outfile (ensOptions.outfile.c_str());

	string seq1_aln,seq2_aln,seq1_aln_str,seq2_aln_str;

	int last_edge_seq1,last_edge_seq2;
	last_edge_seq1=0;
	last_edge_seq2=0;

	for (vector<intPair>::iterator i_edge = matchingsLCSEPM.begin(); i_edge != matchingsLCSEPM.end();++i_edge)
	{
		//cout << myMol1.getId() << "\t" << myMol2.getId() << "\t\t" <<  (*i_edge).first << "\t\t" << (*i_edge).second << "\t" \
			 << myMol1.getBase((*i_edge).first) << "\t" << myMol1.getStructure((*i_edge).first) << myMol2.getBase((*i_edge).second) << "\t" << myMol2.getStructure((*i_edge).second)<< endl;
		//outfile << myMol1.getId() << "\t" << myMol2.getId() << "\t\t" <<  (*i_edge).first << "\t\t" << (*i_edge).second << "\t" \
					 << myMol1.getBase((*i_edge).first) << "\t" << myMol1.getStructure((*i_edge).first) << myMol2.getBase((*i_edge).second) << "\t" << myMol2.getStructure((*i_edge).second)<< endl;

		for (int i=last_edge_seq1+1;i<(*i_edge).first;++i)
		{
			seq1_aln.push_back(myMol1.getBase(i));
			seq2_aln.push_back('-');
			seq1_aln_str.push_back(myMol1.getStructure(i));
			seq2_aln_str.push_back('-');
		}
		for (int j=last_edge_seq2+1;j<(*i_edge).second;++j)
		{
			seq1_aln.push_back('-');
			seq2_aln.push_back(myMol2.getBase(j));
			seq1_aln_str.push_back('-');
			seq2_aln_str.push_back(myMol2.getStructure(j));
		}

		seq1_aln.push_back(myMol1.getBase((*i_edge).first));
		seq2_aln.push_back(myMol2.getBase((*i_edge).second));

		seq1_aln_str.push_back(myMol1.getStructure((*i_edge).first));
		seq2_aln_str.push_back(myMol2.getStructure((*i_edge).second));

		last_edge_seq1= (*i_edge).first;
		last_edge_seq2 = (*i_edge).second;
	}

	// for the part after the last edge
	for (int i=last_edge_seq1+1;i<=myMol1.getLength();++i)
	{
				seq1_aln.push_back(myMol1.getBase(i));
				seq2_aln.push_back('-');
				seq1_aln_str.push_back(myMol1.getStructure(i));
				seq2_aln_str.push_back('-');
			}
	for (int j=last_edge_seq2+1;j<=myMol2.getLength();++j)
	{
		seq1_aln.push_back('-');
		seq2_aln.push_back(myMol2.getBase(j));
		seq1_aln_str.push_back('-');
		seq2_aln_str.push_back(myMol2.getStructure(j));
	}

	//cout << "\n\n Alignment from LCS-EPM:\n" << endl;
	//cout << ">" << myMol1.getId() << endl << seq1_aln << endl << seq1_aln_str << endl;
	//cout << ">" << myMol2.getId() << endl << seq2_aln << endl << seq2_aln_str << endl;
	//outfile << ">" << myMol1.getId() << endl << seq1_aln << endl << seq1_aln_str << endl;
	//outfile << ">" << myMol2.getId() << endl << seq2_aln << endl << seq2_aln_str << endl;
	//outfile << ">" << myMol1.getId() << endl << seq1_aln << endl;
	//outfile << ">" << myMol2.getId() << endl << seq2_aln << endl;
	outfile << "CLUSTAL W (1.83) multiple sequence alignment --- expaRNA 0.5 - exact pattern Alignment of RNA --- Score: "<< matchingsLCSEPM.size() << endl <<endl;

	string tmp1 = myMol1.getId() +"      ";
	string tmp2 = myMol2.getId() +"      ";
	if (tmp1.length() < tmp2.length())
		tmp1.resize(tmp2.length(),' ');
	else
		if (tmp2.length() < tmp1.length())
			tmp2.resize(tmp1.length(),' ');
	string tmp3;
	tmp3.resize(tmp1.length(),' ');
	outfile << tmp3 << seq1_aln_str <<endl;
	outfile << tmp1 << seq1_aln << endl;
	outfile << tmp2 << seq2_aln << endl;
	outfile << tmp3 << seq2_aln_str << endl << endl;
	outfile.close();
}

void	mcsEnsemble::extractMatchings()
{
	std::ifstream fileAlign(ensOptions.cmdLineValues[1].c_str());

	//string dirname = "output";
	//chdir(const_cast<char*>(dirname.c_str()));

	string strRead;
	string alignSeq1,alignSeq2;
	Molecule& myMol1 = ensembleMolecules.find(ensOptions.mol1Id)->second;
	Molecule& myMol2 = ensembleMolecules.find(ensOptions.mol2Id)->second;

	if(fileAlign)
	{
		while(fileAlign)
		{
			strRead.clear();
			while(fileAlign)
			{
				getline(fileAlign,strRead);
				string::size_type loc = strRead.find_first_of("AUTCGautcg-.");
				if( loc != string::npos )
				{
					alignSeq1 += strRead;
					break;
				}
			}
			strRead.clear();
			while(fileAlign)
			{
				getline(fileAlign,strRead);
				string::size_type loc = strRead.find_first_of("AUTCGautcg-.");
				if( loc != string::npos )
				{
					alignSeq2 += strRead;
					break;
				}
			}
		}

		if (alignSeq1.size() != alignSeq2.size() )
		{
			cerr << "Alginment format is wrong. Maybe different sizes!"<<endl;
			exit(1);
		}

		intVec matchingsSeq1Align,matchingsSeq2Align;
		vector<intPair> matchingsAlign;
		int posSeq1,posSeq2;
		posSeq1=1;
		posSeq2=1;

		for (unsigned int i=0;i<alignSeq1.size();++i)
		{
			if ( (alignSeq1[i] == alignSeq2[i]) && (alignSeq2[i]!='-') &&
			     (myMol1.getStructure(posSeq1)== myMol2.getStructure(posSeq2)) &&
			     (myMol1.getBase(posSeq1) == (myMol2.getBase(posSeq2))) )
			{
				matchingsSeq1Align.push_back(posSeq1);
				matchingsSeq2Align.push_back(posSeq2);
				matchingsAlign.push_back(make_pair(posSeq1,posSeq2));

			}
			if (alignSeq1[i] != '-')
				++posSeq1;
			if (alignSeq2[i] != '-')
				++posSeq2;
		}

		cout << "align1:"<< alignSeq1.size()<<endl<< alignSeq1 << endl << endl;
		cout << "align2:"<< alignSeq2.size()<<endl<< alignSeq2 << endl << endl;

		for (unsigned int i=0;i<matchingsSeq1Align.size();++i)
		{
			cout << "match: "<<matchingsSeq1Align[i]<<" with " << matchingsSeq2Align[i]<<endl;
		}

		PatternPairMap myAlignMap;
/*		int act=1;
		int stepSize = 300;

		while (act<matchingsSeq1Align.size())
		{
			string actStr = biu::Util_String::int2str(act);
			int i=1;
			intVec partAlign1,partAlign2;

			while(act<matchingsSeq1Align.size() && (i<=stepSize))
			{
				partAlign1.push_back(matchingsSeq1Align[act]);
				partAlign2.push_back(matchingsSeq2Align[act]);
				++i;++act;
			}
			SinglePattern myAlign1(myMol1.getId(),myMol1.getActualStructureId(),"align"+actStr,partAlign1);
			SinglePattern myAlign2(myMol2.getId(),myMol2.getActualStructureId(),"align"+actStr,partAlign2);

			myAlignMap.add("align"+actStr,matchingsSeq1Align.size(),myAlign1,myAlign2);
		}*/

		intVec matchesSeq1Both;
		intVec matchesSeq2Both;
		intVec matchesSeq1ERPAPOnly;
		intVec matchesSeq2ERPAPOnly;
		intVec matchesSeq1AlignOnly;
		intVec matchesSeq2AlignOnly;
		intVec matchesSeq1ERPAP;
		intVec matchesSeq2ERPAP;
		intVec matchesSeq1ERPAPRest;
		intVec matchesSeq2ERPAPRest;
		intVec matchesSeq1AlignRest;
		intVec matchesSeq2AlignRest;
		intVec matchesSeq1ERPAPAlign;
		intVec matchesSeq2ERPAPAlign;


		// extract matching edges (pairs of positions) from LCS-EPM
		vector<intPair> matchesERPAP;

		for (PatternPairMap::patListCITER i=ensembleMaxERPAP.getList().begin();i != ensembleMaxERPAP.getList().end();i++)
		{
		    matchesSeq1ERPAP.insert(matchesSeq1ERPAP.end(),(*i)->getFirstPat().getPat().begin(),(*i)->getFirstPat().getPat().end());
		    matchesSeq2ERPAP.insert(matchesSeq2ERPAP.end(),(*i)->getSecPat().getPat().begin(),(*i)->getSecPat().getPat().end());
		}
		sort(matchesSeq1ERPAP.begin(),matchesSeq1ERPAP.end());
		sort(matchesSeq2ERPAP.begin(),matchesSeq2ERPAP.end());;

		for (unsigned int i=0;i<matchesSeq1ERPAP.size();++i)
		{
			matchesERPAP.push_back(make_pair(matchesSeq1ERPAP[i],matchesSeq2ERPAP[i]));
		}


		// extract intersecting edges from LCS-EPM with given alignment (from input file)
		vector<int> v(matchesSeq1ERPAP.size());
		vector<int> w(matchesSeq2ERPAP.size());
		vector<intPair> u(matchesSeq1ERPAP.size());
		vector<int>::iterator it1;
		vector<int>::iterator it2;
		vector<intPair>::iterator it3;

		it1 = set_intersection(matchesSeq1ERPAP.begin(),matchesSeq1ERPAP.end(),matchingsSeq1Align.begin(),matchingsSeq1Align.end(),v.begin());
		it2 = set_intersection(matchesSeq2ERPAP.begin(),matchesSeq2ERPAP.end(),matchingsSeq2Align.begin(),matchingsSeq2Align.end(),w.begin());
		it3 = set_intersection(matchesERPAP.begin(),matchesERPAP.end(),matchingsAlign.begin(),matchingsAlign.end(),u.begin());

		//matchesSeq1Both.insert(matchesSeq1Both.end(),v.begin(),it1);
		//matchesSeq2Both.insert(matchesSeq2Both.end(),w.begin(),it2);

		for (vector<intPair>::iterator i=u.begin();i != it3;++i)
		{
			matchesSeq1Both.push_back(i->first);
			matchesSeq2Both.push_back(i->second);
		}
		// rest von ERPAP
		it1 = set_difference(matchesSeq1ERPAP.begin(),matchesSeq1ERPAP.end(),matchesSeq1Both.begin(),matchesSeq1Both.end(),v.begin());
		it2 = set_difference(matchesSeq2ERPAP.begin(),matchesSeq2ERPAP.end(),matchesSeq2Both.begin(),matchesSeq2Both.end(),w.begin());

		matchesSeq1ERPAPRest.insert(matchesSeq1ERPAPRest.end(),v.begin(),it1);
		matchesSeq2ERPAPRest.insert(matchesSeq2ERPAPRest.end(),w.begin(),it2);

		//rest des alignments
		it1 = set_difference(matchingsSeq1Align.begin(),matchingsSeq1Align.end(),matchesSeq1Both.begin(),matchesSeq1Both.end(),v.begin());
		it2 = set_difference(matchingsSeq2Align.begin(),matchingsSeq2Align.end(),matchesSeq2Both.begin(),matchesSeq2Both.end(),w.begin());

		matchesSeq1AlignRest.insert(matchesSeq1AlignRest.end(),v.begin(),it1);
		matchesSeq2AlignRest.insert(matchesSeq2AlignRest.end(),w.begin(),it2);

		// schnittmenge align Rest / ERPAP Rest
		it1 = set_intersection(matchesSeq1ERPAPRest.begin(),matchesSeq1ERPAPRest.end(),matchesSeq1AlignRest.begin(),matchesSeq1AlignRest.end(),v.begin());
		it2 = set_intersection(matchesSeq2ERPAPRest.begin(),matchesSeq2ERPAPRest.end(),matchesSeq2AlignRest.begin(),matchesSeq2AlignRest.end(),w.begin());

		matchesSeq1ERPAPAlign.insert(matchesSeq1ERPAPAlign.end(),v.begin(),it1);
		matchesSeq2ERPAPAlign.insert(matchesSeq2ERPAPAlign.end(),w.begin(),it2);

		// unique rest ERPAP
		it1 = set_difference(matchesSeq1ERPAPRest.begin(),matchesSeq1ERPAPRest.end(),matchesSeq1ERPAPAlign.begin(),matchesSeq1ERPAPAlign.end(),v.begin());
		it2 = set_difference(matchesSeq2ERPAPRest.begin(),matchesSeq2ERPAPRest.end(),matchesSeq2ERPAPAlign.begin(),matchesSeq2ERPAPAlign.end(),w.begin());

		matchesSeq1ERPAPOnly.insert(matchesSeq1ERPAPOnly.end(),v.begin(),it1);
		matchesSeq2ERPAPOnly.insert(matchesSeq2ERPAPOnly.end(),w.begin(),it2);

		// unique rest alignment
		it1 = set_difference(matchesSeq1AlignRest.begin(),matchesSeq1AlignRest.end(),matchesSeq1ERPAPAlign.begin(),matchesSeq1ERPAPAlign.end(),v.begin());
		it2 = set_difference(matchesSeq2AlignRest.begin(),matchesSeq2AlignRest.end(),matchesSeq2ERPAPAlign.begin(),matchesSeq2ERPAPAlign.end(),w.begin());

		matchesSeq1AlignOnly.insert(matchesSeq1AlignOnly.end(),v.begin(),it1);
		matchesSeq2AlignOnly.insert(matchesSeq2AlignOnly.end(),w.begin(),it2);

		SinglePattern myAlign1,myAlign2;

		myAlign1 = SinglePattern(myMol1.getId(),myMol1.getActualStructureId(),"align+ERPAP",matchesSeq1AlignOnly);
		myAlign2 = SinglePattern(myMol2.getId(),myMol2.getActualStructureId(),"align+ERPAP",matchesSeq2AlignOnly);
		myAlignMap.add("align+ERPAP",matchingsSeq1Align.size(),myAlign1,myAlign2);

		myAlign1 = SinglePattern(myMol1.getId(),myMol1.getActualStructureId(),"align+ERPAP",matchesSeq1ERPAPOnly);
		myAlign2 = SinglePattern(myMol2.getId(),myMol2.getActualStructureId(),"align+ERPAP",matchesSeq2ERPAPOnly);
		myAlignMap.add("align+ERPAP",matchingsSeq1Align.size(),myAlign1,myAlign2);

		myAlign1 = SinglePattern(myMol1.getId(),myMol1.getActualStructureId(),"align+ERPAP",matchesSeq1ERPAPAlign);
		myAlign2 = SinglePattern(myMol2.getId(),myMol2.getActualStructureId(),"align+ERPAP",matchesSeq2ERPAPAlign);
		myAlignMap.add("align+ERPAP",matchingsSeq1Align.size(),myAlign1,myAlign2);

		myAlign1 = SinglePattern(myMol1.getId(),myMol1.getActualStructureId(),"align+ERPAP",matchesSeq1Both);
		myAlign2 = SinglePattern(myMol2.getId(),myMol2.getActualStructureId(),"align+ERPAP",matchesSeq2Both);
		myAlignMap.add("align+ERPAP",matchingsSeq1Align.size(),myAlign1,myAlign2);

		for (unsigned int i=0;i<matchesSeq1ERPAPOnly.size();++i)
			cout << "sec: " << matchesSeq1ERPAPOnly[i]<<endl;

		clusterToPS(myMol1,myMol2,"align",matchingsSeq1Align.size(),myAlignMap);

		cout << "# exact matches within reference alignment  : " << matchingsSeq1Align.size() << endl;
		cout << "# exact matches LCS-ERP and alignment (seq1): " << matchesSeq1Both.size() << endl;
		cout << "# exact matches LCS-ERP and alignment (seq2): " << matchesSeq2Both.size() << endl;
		cout << "# matches in seq1 with different position in seq2: " << matchesSeq1ERPAPAlign.size() << endl;
		cout << "# matches in seq2 with different position in seq1: " << matchesSeq2ERPAPAlign.size() << endl;
		cout << "# matches unique to LCS-ERP in seq1  : " << matchesSeq1ERPAPOnly.size() << endl;
		cout << "# matches unique to LCS-ERP in seq2  : " << matchesSeq2ERPAPOnly.size() << endl;
		cout << "# matches unique to alignment in seq1: " << matchesSeq1AlignOnly.size() << endl;
		cout << "# matches unique to alignment in seq2: " << matchesSeq2AlignOnly.size() << endl;
	}
}

/*void	mcsEnsemble::WriteAllToXML()
{
	mcs_xml_io myxml_io("RNAPatterns");
 	myxml_io.xml_append_molecule(ensembleMolecules.find(ensOptions.mol1Id)->second);
 	myxml_io.xml_append_molecule(ensembleMolecules.find(ensOptions.mol2Id)->second);
 	myxml_io.xml_append_patterns(ensemblePatterns);
 	myxml_io.xml_filewriter(ensOptions.mol1Id+"_"+ensOptions.mol2Id+".xml");
}*/
